#!/bin/bash
# Cédric Tabin - http://www.astorm.ch

########################################################################################
# This file contains the definitions of the functions used by the API. It should
# remain untouched !
#
# ===================
# Public functions:
# ===================
#
# showHelp                                   If the switch -h or --help is set, then this function shows the help and exits. The help function is called 'usage'.
#                                            Prior to that, it checks that there is unparset more argument left, otherwise the help is shown and the script is exit.
# readSwitch NAME VARIABLE                   Read the given switch (the name must begin with either '-' or '--'). If the switch is found on the command line, the
#                                            specified variable is set to 1 otherwise to 0.
# readParam NAME VAR1 DEFAULT1 [VAR2 ...]    Read an argument with one or many parameters (ie -n 50) and associate the n-th parameter to the n-th VAR. If the parameter
#                                            is not found, then the variables (VAR1, VAR2, ...) are set to the default one (DEFAULT1, DEFAULT2, ...).
#                                            Note that all arguments are mandatory.
# readLeftParams VAR                         Read all the arguments on the command line that are neither a switch nor a command and put them separated by a space in VAR.
#                                            The variable VAR will be an array.
#
########################################################################################

# This part should remain untouched.
export PARAM_FUNC=1

for k in "$@"; do
  parameters[${#parameters[@]}]="$k"
  read_parameters[${#read_parameters[@]}]=0
done
nb_parameters=${#parameters[@]}

####################
# Public functions #
####################

# showHelp
function showHelp()
{
	checkParams
	if [ "${param_show_help1}" -eq 1 -o "${param_show_help2}" -eq 1 ]; then
		usage
		exit 0
	fi
}

# addSwitch paramName paramVariable
function readSwitch()
{
	p_name="$1"
	p_var="$2"

  is_cmd=`isCommand "${p_name}"`
  is_swt=`isSwitch "${p_name}"`

  if [ ${is_cmd} != "1" -a ${is_swt} != "1" ]; then
    echo "${p_name} is not a switch (neither begins with '-' or '--')"
    exit 2
  fi

	found=0
	for ((i=0 ; i<${nb_parameters} ; i++)); do
    param=${parameters[$i]}
		if [ "${param}" = "${p_name}" ]; then
			found=1
      read_parameters[$i]=1
			export "${p_var}"=1
		fi
	done

	if [ $found -eq 0 ]; then
		export "${p_var}"=0
	fi
}

# readParam paramName var1 defaultValue1 [...]
function readParam()
{
	p_name="$1"
	shift

  if [ `isCommand "${p_name}"` != "1" -a `isSwitch "${p_name}"` != "1" ]; then
    echo "'${p_name}' is not a command (neither begins with '-' or '--')"
    exit 2
  fi

	p_nbreq=`echo $(($# / 2))`
	found=0
	for ((i=0 ; i<${nb_parameters} ; i++)); do
    param=${parameters[$i]}
		if [ "$param" = "${p_name}" ]; then
      read_parameters[$i]=1
			found=1
		elif [ $found -eq 1 -a $# -ne 0 ]; then
			p_var_name="$1"
			p_var_default="$2"
			export "${p_var_name}"="$param"
      read_parameters[$i]=1

      is_cmd=`isCommand "$param"`
      is_swt=`isSwitch "$param"`

			if [ ${is_cmd} = "1" -o ${is_swt} = "1" ]; then
				break
			fi

			shift
			shift
		fi
	done

	if [ $found -eq 1 -a $# -ne 0 ]; then
		echo "Invalid param '${p_name}': at least ${p_nbreq} values required"
		usage
		exit 1
	fi

	if [ $found -eq 0 ]; then
		while [ $# -ne 0 ]; do
			p_var="$1"
			p_default="$2"
			shift
			shift
			export ${p_var}="${p_default}"
		done
	fi
}

# readLeftParams paramName 
function readLeftParams()
{
	p_name="$1"

	for ((i=$nb_parameters-1 ; i>=0 ; i--)); do
    param=${parameters[$i]}

    if [ ${read_parameters[$i]} -eq 1 ]; then
      break
    fi
    
    is_cmd=`isCommand "$param"`
    is_swt=`isSwitch "$param"`
  
		if [ ${is_cmd} = "0" -a ${is_swt} = "0" ]; then
      size=${#left_params[@]}
			left_params[$size]="$param"
      read_parameters[$i]=1
		fi
	done

	nb_left_params=${#left_params[@]}
  for ((i=0 ; i<$nb_left_params ; i++)); do
      last=$(($nb_left_params - 1 - $i))
      eval "${p_name}[$i]=\"${left_params[$last]}\""
  done
  
  export ${p_name}
}

#####################
# Private functions #
#####################

# isSwitch "-p"
function isSwitch()
{
	val="$1"
	if [[ "$val" == --* ]]; then
		echo "0"
  elif [[ "$val" == -* ]]; then
    echo "1"
	else
		echo "0"
	fi
}

# isCommand "--help"
function isCommand()
{
  val="$1"
	if [[ "$val" == --* ]]; then
		echo "1"
  else
		echo "0"
	fi
}

function checkParams()
{
  for ((i=0 ; i<$nb_parameters ; i++)); do
    if [ ${read_parameters[$i]} -eq 0 ]; then
      echo "Unrecognized parameter: '${parameters[$i]}'"
      usage
      exit 1
    fi
  done
}
